/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XPath Checker.
 *
 * The Initial Developer of the Original Code is Brian Slesinsky.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Brian Slesinsky <brian@slesinsky.org> (Original Author)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function getXPath(targetNode) {
    var useLowerCase = (targetNode.ownerDocument instanceof HTMLDocument)
    var nodePath = getNodePath(targetNode)
    var nodeNames = []
    for (var i in nodePath) {
        var nodeIndex
        var node = nodePath[i]
        if (node.nodeType == 1) { // && node.tagName != "TBODY") {
            if (i == 0 && node.hasAttribute("id")) {
                nodeNames.push("/*[@id='" + node.getAttribute("id") + "']");
            } else {
                var tagName = node.tagName
                if (useLowerCase) {
                    tagName = tagName.toLowerCase()
                }
                nodeIndex = getNodeIndex(node)
                if (nodeIndex != null) {
                    nodeNames.push(tagName + "[" + nodeIndex + "]");
                } else {
                    nodeNames.push(tagName);
                }
            }
        } else if (node.nodeType == 3) {
            nodeIndex = getTextNodeIndex(node)
            if (nodeIndex != null) {
                nodeNames.push("text()[" + nodeIndex + "]");
            } else {
                nodeNames.push("text()");
            }
        }
    }
    return "/" + nodeNames.join("/")

}

function getNodeIndex(node) {
    if (node.nodeType != 1 || node.parentNode == null) return null
    var list = getChildNodesWithTagName(node.parentNode, node.tagName)
    if (list.length == 1 && list[0] == node) return null
    for (var i = 0; i < list.length; i++) {
        if (list[i] == node) return i + 1
    }
    throw "couldn't find node in parent's list: " + node.tagName
}

function getTextNodeIndex(node) {
    var list = getChildTextNodes(node.parentNode)
    if (list.length == 1 && list[0] == node) return null
    for (var i = 0; i < list.length; i++) {
        if (list[i] == node) return i + 1
    }
    throw "couldn't find node in parent's list: " + node.tagName
}

function getChildNodesWithTagName(parent, tagName) {
    var result = []
    var child = parent.firstChild
    while (child != null) {
        if (child.tagName && child.tagName == tagName) {
            result.push(child)
        }
        child = child.nextSibling
    }
    return result
}

function getChildTextNodes(parent) {
    var result = []
    var child = parent.firstChild
    while (child != null) {
        if (child.nodeType==3) {
            result.push(child)
        }
        child = child.nextSibling
    }
    return result
}

function getNodePath(node) {
    var result = []
    while (node.nodeType == 1 || node.nodeType == 3) {
        result.unshift(node)
        if (node.nodeType == 1 && node.hasAttribute("id")) return result
        node = node.parentNode
    }
    return result
}
